@@ -6,7 +6,12 @@ import asyncio
 import random
 import struct
 
-from urllib.parse import urlsplit
+from pyslot import Signal
+
+from urllib.parse import (
+    urlsplit,
+    urlunsplit,
+)
 from contextlib import ExitStack
 from functools import partial
 
@@ -148,6 +153,10 @@ class Socket(CompositeClosableAsyncObject):
         else:
             raise ValueError("Unsupported socket type: %r" % self.socket_type)
 
+        self.on_connection_ready = Signal()
+        self.on_connection_lost = Signal()
+        self.on_connection_failure = Signal()
+
     def create_inbox(self):
         return AsyncBox(
             maxsize=self.max_inbox_size,
@@ -160,7 +169,7 @@ class Socket(CompositeClosableAsyncObject):
             loop=self.loop,
         )
 
-    def connect(self, endpoint, *, on_connection_failure=None):
+    def connect(self, endpoint):
         url = urlsplit(endpoint)
 
         if url.scheme == 'tcp':
@@ -202,8 +211,9 @@ class Socket(CompositeClosableAsyncObject):
             partial(self.unregister_connection, engine=engine),
         )
 
-        if on_connection_failure:
-            engine.on_connection_failure.connect(on_connection_failure)
+        engine.on_connection_ready.connect(self.on_connection_ready.emit)
+        engine.on_connection_lost.connect(self.on_connection_lost.emit)
+        engine.on_connection_failure.connect(self.on_connection_failure.emit)
 
         self._outgoing_engines[url] = engine
         peer = Peer(
@@ -221,11 +231,29 @@ class Socket(CompositeClosableAsyncObject):
 
         engine = self._outgoing_engines.pop(url)
         peer = self._outgoing_peers.pop(engine)
-        self._peers.remove(peer)
+
+        if peer.connection:
+            # If the peer is connected, we just close the connection and let
+            # the usual clean-up take place.
+            peer.connection.close()
+        else:
+            # If the peer does not have a connection, we can remove it safely
+            # as it will never have one (and if the connection were to succeed
+            # anyway, it would create then destroy a new peer).
+            self._peers.remove(peer)
+
         engine.close()
         await engine.wait_closed()
 
-    def bind(self, endpoint, *, on_connection_failure=None):
+    async def reconnect(self, endpoint):
+        await self.disconnect(endpoint)
+        self.connect(endpoint)
+
+    async def reconnect_all(self):
+        for endpoint in list(self._outgoing_engines):
+            await self.reconnect(urlunsplit(endpoint))
+
+    def bind(self, endpoint):
         url = urlsplit(endpoint)
 
         if url.scheme == 'tcp':
@@ -267,8 +295,9 @@ class Socket(CompositeClosableAsyncObject):
             partial(self.unregister_connection, engine=engine),
         )
 
-        if on_connection_failure:
-            engine.on_connection_failure.connect(on_connection_failure)
+        engine.on_connection_ready.connect(self.on_connection_ready.emit)
+        engine.on_connection_lost.connect(self.on_connection_lost.emit)
+        engine.on_connection_failure.connect(self.on_connection_failure.emit)
 
         self._incoming_engines[url] = engine
         self.register_child(engine)
@@ -280,6 +309,18 @@ class Socket(CompositeClosableAsyncObject):
         engine.close()
         await engine.wait_closed()
 
+    async def rebind(self, endpoint):
+        await self.unbind(endpoint)
+        self.bind(endpoint)
+
+    async def rebind_all(self):
+        for endpoint in list(self._incoming_engines):
+            await self.rebind(urlunsplit(endpoint))
+
+    async def reset_all(self):
+        await self.reconnect_all()
+        await self.rebind_all()
+
     def register_connection(self, connection, engine):
         logger.debug("Registering new active connection: %s", connection)
 
